︠7a696e8b-f907-4052-b331-7fc32e68a98ai︠
%html
Here is a very basic code to see what tangent words are, and their relation with balanced words (Sturmian factors).

︡eb035835-25d3-4c5d-843c-ec6f19b0c1f5︡{"html": "Here is a very basic code to see what tangent words are, and their relation with balanced words (Sturmian factors)."}︡
︠d1d99288-667c-4d29-a8c2-7c5274a0a798︠
def sturmian_desubstitute_as_possible(self):
    r"""
    Sturmian desubstitutes the word self as much as possible.

    The word self must be definied on a two-letter alphabet. It can be
    Sturmian desubstituted if one letter appears isolated. The
    Sturmian desubstitution consist in removing the shortest run to
    any run of the non-isolated letter. The Sturmian desubstitution is
    done as much as possible. A word is a factor of a Sturmian word
    if, and only if, the result is the empty word.
    """
    # print self
    W = self.parent()
    if (W.size_of_alphabet() != 2):
        raise TypeError, 'Your word must be defined on a binary alphabet'
    alphabet = W.alphabet()
    if self.is_empty():
        return self
    is_prefix = True
    current_run_length = 0
    prefix_length = 0
    prefix_letter = self[0]
    is_isolated = {alphabet[0]:True,alphabet[1]:True}
    minimal_run = {alphabet[0]:Infinity,alphabet[1]:Infinity}
    maximal_run = {alphabet[0]:0,alphabet[1]:0}
    runs = {alphabet[0]:[],alphabet[1]:[]}
    for i in self:
        if is_prefix:
            if i == prefix_letter:
                prefix_length += 1
                if prefix_length > 1:
                    is_isolated[i] = False
            else:
                is_prefix = False
                current_run_length = 1
                previous_letter = i
        else:
            if i == previous_letter:
                current_run_length += 1
                if current_run_length > 1:
                    is_isolated[i] = False
            else:
                runs[previous_letter].append(current_run_length)
                minimal_run[previous_letter] = min(minimal_run[previous_letter],current_run_length)
                maximal_run[previous_letter] = max(maximal_run[previous_letter],current_run_length)
                current_run_length = 1
                previous_letter = i
    # print runs
    # at his point, previous_letter is the suffix letter and current_run_length is the suffix length
    if not (is_isolated[alphabet[0]] or is_isolated[alphabet[1]]):
        return self
    elif is_isolated[alphabet[0]] and is_isolated[alphabet[1]]:
        return W()
    else:
        if is_isolated[alphabet[0]]:
            l_isolated = alphabet[0] #the isolated letter
            l_running = alphabet[1] #the running letter (non-isolated)
        else:
            l_isolated = alphabet[1]
            l_running = alphabet[0]
        w_isolated = W(l_isolated,datatype='letter') #the word associated to the isolated letter
        w_running = W(l_running,datatype='letter') #the word associated to the running letter
        min_run = minimal_run[l_running]
        if (prefix_letter == l_isolated) or (prefix_length <= min_run):
            desubstitued_word = W()
        else:
            desubstitued_word = w_running ** (prefix_length - min_run)
        for i in runs[l_running]:
            desubstitued_word = desubstitued_word + w_isolated + w_running ** (i - min_run)
        if (current_run_length > 0):
            desubstitued_word = desubstitued_word + w_isolated
            if (previous_letter == l_running) and (current_run_length > min_run):
                desubstitued_word = desubstitued_word + w_running ** (current_run_length - min_run)
        return sturmian_desubstitute_as_possible(desubstitued_word)
︡f1e72a34-1407-4a77-bac6-38b3658fb058︡︡
︠485c8954-0bd0-43d5-9f0b-635571d8967ai︠
%html
Some examples

︡53ebe571-e211-45f8-a8d8-24421df8db4f︡{"html": "Some examples"}︡
︠85249114-08a7-4a46-9961-af6cda2211d7︠
u = Word('10111101101110111',alphabet='01') ; u
v = sturmian_desubstitute_as_possible(u) ; v
︡24ea6522-8a7e-4456-b72a-ce1ba409b61c︡{"stdout": "word: 01100101"}︡
︠6c3fda42-bae5-46d2-add3-47ce71dde6af︠
sturmian_desubstitute_as_possible(Word('azaazaaazaaazaazaaaz', alphabet='az'))
︡a592da11-1e14-4bc5-bb81-1e22c9e4f0b2︡{"stdout": "word:"}︡
︠60ad56cb-35ef-411b-b0f6-7aafd7a57a81︠
def is_sturmian_factor(self):
    r"""
    Tells whether self is a factor of a Sturmian word.

    Equivalently, tells whether self is balanced. The advantage over
    the is_balanced() method is that this one runs in linear time
    whereas is_balanced() runs in quadratic time. The word self must
    be definied on a two-letter alphabet.
    """
    return sturmian_desubstitute_as_possible(self).is_empty()
︡9dd7a7e0-cdc6-4d7d-9336-90d73343a36f︡︡
︠8c6bd7c2-2aab-4f4b-8d5d-0b7ae94cfbf1i︠
%html
An example

︡404287c0-2473-4dee-ba76-e3336804ca52︡{"html": "An example"}︡
︠061636a0-cc14-4038-9813-a8607935da9b︠
w = Word('0111011011011101101',alphabet='01')
is_sturmian_factor(w)
︡c26d8050-e0ee-46a7-860e-c13895b705b1︡{"stdout": "True"}︡
︠08d4a9a9-9e13-4ed7-b8e0-1c4d55eac2dfi︠
%html
Fit with existing classes.

︡3a307d2c-87a7-4d58-b3e2-831e78bdcfb6︡{"html": "Fit with existing classes."}︡
︠1c0434d4-5105-4141-8c6f-e7cab17c713c︠
print is_sturmian_factor(words.LowerMechanicalWord(random(),alphabet='01')[:100])
print is_sturmian_factor(words.CharacteristicSturmianWord(random())[:100])
︡f92f818e-a236-4280-9a2c-ff9c53116ef3︡{"stdout": "True\nTrue"}︡
︠f1918d62-fe8f-438c-abff-9726d4d156f2i︠
%html
Famous words

︡62c07e87-61a2-4449-a030-a4b0adbbe50f︡{"html": "Famous words"}︡
︠79e43f1d-fa45-4122-b3f8-4727e4a02368︠
is_sturmian_factor(words.FibonacciWord()[:100])
︡1c228e9c-b03b-4e00-ab1a-3237cad8e102︡{"stdout": "True"}︡
︠972bdc0d-5e20-45e6-92a7-9bf974318555︠
is_sturmian_factor(words.ThueMorseWord()[:1000])
︡7aeb7595-9f98-4acc-b074-84d236179b86︡{"stdout": "False"}︡
︠1a226c0f-0519-429d-b125-075f46d75313︠
# The Kolakoski word will be soon realeased in sage
#is_sturmian_factor(words.KolakoskiWord()[:1000])
︡a1320f30-e484-4365-88ee-9708cf4cda16︡︡
︠2c490015-a1a5-4175-a249-fa548fe383fd︠
def is_tangent(self):
    r"""
    Tells whether self is a tangent word.

    A binary word is said to be tangent if it can appear in the
    cutting sequences of a smooth curve to arbitrary small scale.
    This class of words strictly contains the class of 1-balanced
    words, and is strictly contained in the class of 2-balanced words.

    This method runs in linear time.
    """
    if (self.parent().size_of_alphabet() != 2):
        raise TypeError, 'Your word must be defined on a binary alphabet '
    [a,b] = self.parent().alphabet()
    mini = 0
    maxi = 0
    height = 0
    for i in sturmian_desubstitute_as_possible(self):
        if i == a:
            height = height + 1
            maxi = max(maxi , height)
        if i == b:
            height = height - 1
            mini = min(mini , height)
    return (maxi - mini <= 2)
︡a3288cc2-e0fb-44f5-aafb-556e6a5ddfed︡︡
︠ca4ffa2a-2afb-41f2-9dab-dd37c34ee81di︠
%html
An example

︡f41d9fb7-03ed-47a8-a0e2-ff2f007fac9b︡{"html": "An example"}︡
︠852cfa4c-0219-406b-80ea-348bf462afe9︠
w = Word('01110110110111011101',alphabet='01')
is_tangent(w)
︡8c01c648-1d49-461e-b7e4-8748f9af350d︡{"stdout": "True"}︡
︠ed81268f-fe1d-4aab-bc49-e3fe9a706917i︠
%html
Some tangent words may not be balanced

︡fd42eee9-1e17-4d57-934a-091ef2fada70︡{"html": "Some tangent words may not be balanced"}︡
︠11f6de34-51ad-41f4-a9c6-7ed2f7a778dc︠
print Word('aabb',alphabet='ab').is_balanced()
print is_tangent(Word('aabb',alphabet='ab'))
︡bc4586aa-41a3-44ca-9def-8ef33831f3b7︡{"stdout": "False\nTrue"}︡
︠a2548f0b-1e21-402e-9b55-df816a11b2dbi︠
%html
Some 2-balanced words may not be tangent

︡c51d744c-56f0-4fa0-aafd-3d73fbd0364f︡{"html": "Some 2-balanced words may not be tangent"}︡
︠4a0f7246-f7df-4342-90bf-52e700e70cd7︠
print is_tangent(Word('aaabb',alphabet='ab'))
print Word('aaabb',alphabet='ab').is_balanced(2)
︡10764e2d-e978-4e3d-a5c2-3fa83119a264︡{"stdout": "False\nTrue"}︡
︠0771515b-9afd-4c33-b34b-4af4178034eci︠
%html
Famous examples

︡d099646a-fb97-402a-a88d-ced727d77afd︡{"html": "Famous examples"}︡
︠c7f7278e-8b94-47c9-a157-65cded7f8738︠
is_tangent(words.FibonacciWord()[:100])
︡ba5a9243-5ff6-4e33-8c12-f56d084d3e8f︡{"stdout": "True"}︡
︠6fad11ba-16af-4168-b7fa-f639a67a25a8︠
is_tangent(words.ThueMorseWord()[:1000])
︡8ab0a89b-01e1-4311-b783-327dbdf36b9c︡{"stdout": "True"}︡
︠e97ce0fa-b790-45bc-a2f8-567d8cb553e6︠
# The Kolakoski word will be soon realeased in sage
#is_tangent(words.KolakoskiWord()[:1000])
︡1dfa9e4d-d9bb-4b28-be9b-7167e998085e︡︡
